@@ -1,4 +1,4 @@
-/* $Id: ttt_tk.h,v 0.1 1996/06/30 12:52:32 kjc Exp $ */
+/* $Id: ttt_tk.h,v 0.2 2003/10/16 10:38:32 kjc Exp $ */
 /* ttt_tk.h -- tk-related header */
 #ifndef _TTT_TK_H_
 #define _TTT_TK_H_
@@ -6,6 +6,11 @@
 #include "tk.h"
 #include "blt.h"
 
+/* const directive for tcl84 */
+#ifndef CONST84
+#define CONST84
+#endif
+
 extern Tcl_Interp *ttt_interp;		/* Interpreter for application. */
 extern Tk_FileProc net_read;		/* read proc for pcap */
 
